﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using Telerik.Windows.Controls.DragDrop;
using Telerik.Windows;
using System.Collections;
using System.Diagnostics;

namespace SilverlightApplication1
{
	public partial class MainPage : UserControl
	{
		private Point mousePosition;

		public MainPage()
		{
			InitializeComponent();

			//Wire the events:
			canvas1.AddHandler(RadDragAndDropManager.DropQueryEvent, new EventHandler<DragDropQueryEventArgs>(OnDropQuery));
			canvas1.AddHandler(RadDragAndDropManager.DropInfoEvent, new EventHandler<DragDropEventArgs>(OnDropInfo));

			listBox1.AddHandler(RadDragAndDropManager.DragInfoEvent, new EventHandler<DragDropEventArgs>(OnDragInfo));
			listBox1.AddHandler(RadDragAndDropManager.DragQueryEvent, new EventHandler<DragDropQueryEventArgs>(OnDragQuery));
		}

		private void OnDropInfo(object sender, DragDropEventArgs e)
		{
			var destination = e.Options.Destination;

			if (e.Options.Status == DragStatus.DropComplete && destination is Canvas)
			{
				// e.Options.Payload is the string which is the name of the desired UserControl.
				string controlName = e.Options.Payload.ToString();
				// Depending on the name of the UserControl, we can return a new instance of that UserControl.
				UserControl userControl = this.GetDesiredUserControl(controlName);
				// Add it to the host canvas.
				canvas1.Children.Add(userControl);
				// Determine the drop point for the new UserControl.
				Point canvasPosition = canvas1.TransformToVisual(null).Transform(new Point());
				Point userControlDesiredPosition = new Point();
				// e.Options.CurrentDragPoint returns the absolute mouse position,
				// that is why we have to transform the host canvas and subtract its position from the absolute mouse position.
				userControlDesiredPosition.X = e.Options.CurrentDragPoint.X - canvasPosition.X;
				userControlDesiredPosition.Y = e.Options.CurrentDragPoint.Y - canvasPosition.Y;
				userControl.SetValue(Canvas.LeftProperty, userControlDesiredPosition.X);
				userControl.SetValue(Canvas.TopProperty, userControlDesiredPosition.Y);
			}
		}

		/// <summary>
		/// Gets a new instance of a UserControl, depending on its name
		/// </summary>
		/// <param name="userControlName">Name of the user control.</param>
		/// <returns></returns>
		private UserControl GetDesiredUserControl(string userControlName)
		{
			if (userControlName == "TreeView")
			{
				return new TreeViewUserControl();
			}
			else if (userControlName == "Ellipse")
			{
				return new EllipseUserControl();
			}
			else if (userControlName == "Book")
			{
				return new BookUserControl();
			}
			else if (userControlName == "Rectangle")
			{
				return new RectangleUserControl();
			}
			else
			{
				return new GenericUserControl();
			}
		}
		private void OnDropQuery(object sender, DragDropQueryEventArgs e)
		{
			var destination = e.Options.Destination;

			if (e.Options.Status == DragStatus.DropDestinationQuery && destination is Canvas)
			{
				e.QueryResult = true;
				e.Handled = true;
			}
		}
		private void OnDragInfo(object sender, DragDropEventArgs e)
		{
			if (e.Options.Status == DragStatus.DragComplete)
			{
				// If you want to remove the dragged item from the listbox, you can uncomment the line bellow.
				//listBox1.Items.Remove(e.Options.Source);
			}
		}
		private void OnDragQuery(object sender, DragDropQueryEventArgs e)
		{
			if (e.Options.Status == DragStatus.DragQuery)
			{
				var draggedListBoxItem = e.Options.Source as ListBoxItem;
				e.Options.DragCue = draggedListBoxItem.Content;
				e.Options.Payload = draggedListBoxItem.Content;
			}

			e.QueryResult = true;
			e.Handled = true;
		}
	}
}
